

// A simple program to test the Tree methods

// M. A. Sridhar, May 28, 1997

#include "base/tree.h"
#include "base/treewalk.h"
#include "base/memory.h"


#include <iostream.h>


void AddChild (CL_ObjectPtrTree& aTree, const char* treeName,
               long childLabel, long parentLabel)
{
    CL_String nodeName (treeName);
    aTree.AddChild (childLabel, parentLabel)->SetContent
        (new CL_String
         (CL_String ("'") + nodeName + "' node " + CL_String (childLabel)));
}


void BuildTree (CL_ObjectPtrTree& aTree, const char* treeName, long startLabel)
{
    CL_String nodeName (treeName);
    aTree.NewRoot (startLabel)->SetContent
        (new CL_String
         (CL_String ("'") + nodeName + "' node " + CL_String (startLabel)));
    AddChild (aTree, treeName, startLabel+1, startLabel);
    AddChild (aTree, treeName, startLabel+2, startLabel);
    AddChild (aTree, treeName, startLabel+3, startLabel);

    AddChild (aTree, treeName, startLabel+4, startLabel+2);
    AddChild (aTree, treeName, startLabel+5, startLabel+2);

}


void DestroyContents (CL_ObjectPtrTree& aTree)
{
    CL_ObjectPtrTreePostWalker walker (aTree.Root());
    while (walker.More()) {
        CL_Object* p = walker.Next()->Content();
        delete p;
    }
}


void main ()
{
    CL_MemoryLeakChecker ck (cout);

    CL_ObjectPtrTree aTree;
    BuildTree (aTree, "Main tree", 1);
    cout << "Main tree:"
         << "\n--------\n" << aTree
         << "\n--------\n\n";

    CL_ObjectPtrTree tree2;
    BuildTree (tree2, "Other tree", 10);
    aTree.GraftSubtree (3, tree2, 0);
    cout << "After graft:"
         << "\n--------\n" << aTree
         << "\n--------\n\n";
    cout << "Other tree after graft (should be empty):"
         << "\n--------\n" << tree2
         << "\n--------\n\n";

    tree2 = aTree;
    cout << "Other tree after assignment:" 
         << "\n--------\n" << tree2
         << "\n--------\n\n";
    
    DestroyContents (aTree);

    // Note that we don't destroy tree2's contents: the pointers are the
    // same as those in aTree!
}

    

