


/*
 *
 *          Copyright (C) 1997, M. A. Sridhar
 *  
 *
 *     This software is Copyright M. A. Sridhar, 1997. You are free
 *     to copy, modify or distribute this software  as you see fit,
 *     and to use  it  for  any  purpose, provided   this copyright
 *     notice and the following   disclaimer are included  with all
 *     copies.
 *
 *                        DISCLAIMER
 *
 *     The author makes no warranties, either expressed or implied,
 *     with respect  to  this  software, its  quality, performance,
 *     merchantability, or fitness for any particular purpose. This
 *     software is distributed  AS IS.  The  user of this  software
 *     assumes all risks  as to its quality  and performance. In no
 *     event shall the author be liable for any direct, indirect or
 *     consequential damages, even if the  author has been  advised
 *     as to the possibility of such damages.
 *
 */


#include "ui/support/x_yacl/stredevt.h"
#include "ui/event.h"
#include "ui/stred.h"
#include "ui/dsplsurf.h"
#include "ui/dsinmem.h"
#include "ui/applic.h"
#include "ui/cntroler.h"
#include "ui/shadorec.h"
#include "ui/dsinmem.h"

#include <iostream.h>

#include <X11/keysym.h>

typedef CL_Binding0<YVE_StringEditorEventHandler> Bind0;
typedef UI_EventBinding1<YVE_StringEditorEventHandler> Bind1;

#if defined(__GNUC__)
template class CL_Binding0<YVE_StringEditorEventHandler>;
template class UI_EventBinding1<YVE_StringEditorEventHandler>;
#endif


const short BLINK_INTERVAL = 500; // # milliseconds per blink

YVE_StringEditorEventHandler::YVE_StringEditorEventHandler
    (UI_StringEditor* editor)
: _edit (editor),
  _mouseDownPt (-1, -1),
  _visibleStartIndex (0),
  _selectStartIndex (-1),
  _blinkTimer (Bind0 (this, YVE_StringEditorEventHandler::_BlinkTimerTick))
{
    Bind1 bind (this, &YVE_StringEditorEventHandler::_HandleEvent );
    _edit->AddEventDependent (Event_KeyTyped,        bind);
    _edit->AddEventDependent (Event_GetFocus,        bind);
    _edit->AddEventDependent (Event_LoseFocus,       bind);
    _edit->AddEventDependent (Event_Paint,           bind);
    _edit->AddEventDependent (Event_LButtonPress,    bind);
    _edit->AddEventDependent (Event_MouseMove,       bind);
    _edit->AddEventDependent (Event_LButtonRelease,  bind);
    _haveFocus = YACLApp()->Controller().Focus() == _edit ? TRUE : FALSE;
    _inhibitRedraw = FALSE;
}


YVE_StringEditorEventHandler::~YVE_StringEditorEventHandler ()
{
    if (_blinkTimer.IsRunning())
        _blinkTimer.Stop();
}

bool YVE_StringEditorEventHandler::_HandleEvent (UI_Event& evt)
{
    UI_EventType type = evt.Type ();
    switch (type) {
    case Event_KeyTyped:
        _DoKeyTyped (evt);
        break;

    case Event_GetFocus:
        _DoFocus (TRUE);
        break;
        
    case Event_LoseFocus:
        _DoFocus (FALSE);
        break;

    case Event_Paint:
        _DoPaint ();
        break;

    case Event_LButtonPress:
        _DoMouseDown (evt);
        break;

    case Event_MouseMove:
        _DoMouseMove (evt);
        break;
        
    case Event_LButtonRelease:
        _DoMouseUp (evt);
        break;
    }
        
    return TRUE;
}


bool YVE_StringEditorEventHandler::_DoInsert (short key)
{
    CL_String& data = (CL_String&) _edit->Model();
    long limit = _edit->LengthLimit();
    if (limit < 0 || data.Size() < limit) {
        CL_Integer& pos = _edit->InsertionPosition ();
        if (_edit->FilterChar (key)) {
            _inhibitRedraw = TRUE; // Don't repaint in response to the
                                   // change in the model below
            data.Insert (key, pos-1);
            _inhibitRedraw = FALSE; // Now repaint
            pos++;
            // One of these days I'll fix it so that only one paint is
            // generated. -- MAS 5/26/97
            _edit->Update();
        }
        return TRUE;
    }
    YACLApp()->Controller().Beep();
    return FALSE;
}


void YVE_StringEditorEventHandler::_DoKeyTyped (UI_Event& evt)
{
    ushort key = evt.Key ();
    CL_Integer& pos = _edit->InsertionPosition();
    CL_String& data = (CL_String&) _edit->Model();
    switch (key) {
    case UI_KeyboardEvent::Key_RightArrow:
        pos = minl (1 + pos, data.Size());
        break;

    case UI_KeyboardEvent::Key_LeftArrow:
        pos = maxl (pos - 1, 0);
        break;

    case UI_KeyboardEvent::Key_End:
        pos = data.Size();
        break;

    case UI_KeyboardEvent::Key_Home:
        pos = 0;
        break;

    case UI_KeyboardEvent::Key_Delete: {
        CL_Interval& selection = _edit->Selection();
        if (selection.High() >= 0) {
            _inhibitRedraw = TRUE;
            long lo = selection.Low();
            data (lo, selection.Length()) = "";
            pos = lo;
            _inhibitRedraw = FALSE;
            selection = CL_Interval (-1, -1);
        }
        else if (pos < data.Size())
            data.Delete (pos);
        break;
    }

    case UI_KeyboardEvent::Key_Backspace: {
        CL_Interval& selection = _edit->Selection();
        if (selection.High() > 0) {
            _inhibitRedraw = TRUE;
            long lo = selection.Low();
            data (lo, selection.Length()) = "";
            pos = lo;
            _inhibitRedraw = FALSE;
            selection = CL_Interval (-1, -1);
        }
        else if (pos <= data.Size() && pos > 0) {
            data.Delete (pos - 1);
            pos--;
        }
        break;
    }
    
    default:
        if (key >= 32 && key < 127) { // It's a printable character
            _DoInsert (key);
        }
        break;
    }
}


long YVE_StringEditorEventHandler::_IndexOfCharAt (const UI_Point& pt)
{
    long ptX = pt.XCoord();
    CL_String& model = (CL_String&) _edit->Model();
    const char* data = model.AsPtr();
    data += _visibleStartIndex;
    XFontStruct* fontPtr = (XFontStruct*) _edit->_font->NativeFontStruct();
    long i;
    for (i = 0; data[i] != 0; i++) {
        long offset = XTextWidth (fontPtr, data, i+1);
        if (offset > ptX)
            break;
    }
    return minl (i + _visibleStartIndex, model.Size());
}

void YVE_StringEditorEventHandler::_DoMouseDown (UI_Event& evt)
{
    CL_Interval& selection = _edit->Selection();
    _mouseDownPt = evt.Position ();
    selection = CL_Interval (-1, -1); // Repaint
    _selectStartIndex = _IndexOfCharAt (_mouseDownPt);
    _mouseGrabbed = FALSE;
    if (_blinkTimer.IsRunning())
        _blinkTimer.Stop();
}

void YVE_StringEditorEventHandler::_DoMouseMove (UI_Event& evt)
{
    if (_mouseDownPt != UI_Point (-1, -1)) {
        // The user is making  or extending a selection
        if (!_mouseGrabbed) {
            ulong mask = ButtonPressMask | ButtonReleaseMask | ButtonMotionMask
                | PointerMotionMask ;
            XGrabPointer (YACLApp()->AppDisplay(), _edit->ViewHandle(), False, 
                          mask, GrabModeAsync, GrabModeAsync, None, None,
                          CurrentTime);
            _mouseGrabbed = TRUE;
            _edit->CreateDisplaySurface ();
        }
        long endIndex = minl (_IndexOfCharAt (evt.Position ()),
                              ((CL_String*) _edit->_model)->Size()-1);
        CL_Interval& oldSelection = _edit->Selection();
        CL_Interval newSelection (_selectStartIndex, endIndex);
        if (oldSelection != newSelection && newSelection.Length() > 0) {
            _InvertCharRange (_edit->DisplaySurface(), oldSelection);
            oldSelection = newSelection;
            _InvertCharRange (_edit->DisplaySurface(), newSelection);
        }
    }
}

void YVE_StringEditorEventHandler::_DoMouseUp (UI_Event& evt)
{
    if (evt.Position() == _mouseDownPt) {
        long index = _IndexOfCharAt (_mouseDownPt);
        _edit->InsertionPosition() = index;
    }
    else {
        _edit->InsertionPosition() = _IndexOfCharAt (evt.Position());
        XUngrabPointer (YACLApp()->AppDisplay(), CurrentTime);
        _mouseGrabbed = FALSE;
    }        
    _mouseDownPt = UI_Point (-1, -1);
    if (_blinkTimer.IsRunning())
        _blinkTimer.Stop();
    _blinkTimer.Start (BLINK_INTERVAL);
}

void YVE_StringEditorEventHandler::_DoFocus (bool gotFocus)
{
    _haveFocus = gotFocus;
    if (gotFocus) {
        _DoPaint ();
        _blinkTimer.Start (BLINK_INTERVAL);
    }
    else {
        _blinkTimer.Stop();
        _blinkOn = FALSE;
        _DoPaint ();
    }
}



void YVE_StringEditorEventHandler::_DoPaint ()
{
    bool dsp = FALSE;
    UI_DisplaySurface* sfc = _edit->DisplaySurface ();
    if (!sfc) {
        dsp = TRUE;
        sfc = &(_edit->CreateDisplaySurface ());
    }
    UI_Rectangle r = _edit->Shape();
    r.Origin (UI_Point (0, 0));
    long w = r.Width(), h = r.Height();
    UI_DwgSurfaceInMemory memDS (w + 1, h + 1, sfc);
    memDS.ColorRectangle (r, _edit->Background());
    UI_Color borderColor (UIColor_Black);
    if (!_haveFocus)
        borderColor = _edit->Foreground().LightShadow();
    UI_Rectangle border (0, 0, w, h);
    memDS.Pen().Color (borderColor);
    memDS.DrawRectangle (border, UID_Outline);

    UI_ShadowRectangle shape (0, 0, w, h, UI_ShadowRectangle::Recessed, 1,
                              _haveFocus);
    shape.DrawOn (memDS);
    UI_Rectangle interior = shape.Interior();
    memDS.ColorRectangle (interior, _edit->Background());

    memDS.Pen().Color (_edit->Foreground());
    CL_String& data = (CL_String&) _edit->Model();
    memDS.WriteString (data.Suffix (_visibleStartIndex), interior,
                       UIText_Left);

    // Invert the selection, if any
    short fontHeight = sfc->Font().Height();
    CL_Interval& selection = _edit->Selection();
    if (selection.High() >= 0) {
        _InvertCharRange (&memDS, selection);
    }

    // Draw the blinking caret if necessary
    if (_blinkOn) {
        XFontStruct* fontPtr = (XFontStruct*) memDS.Font().NativeFontStruct();
        const char* dataPtr = data.AsPtr();
        short offset = XTextWidth
            (fontPtr, dataPtr, _edit->InsertionPosition()) + 2;
        // --------------------------------------------------^--
        // Allow for the shadow
        memDS.Pen().Color (UIColor_Black);
        memDS.DrawLine (UI_Point (offset, 2),
                        UI_Point (offset,  fontHeight + 1));
    }
    memDS.CopyTo (*sfc);
    if (dsp)
        _edit->DestroyDisplaySurface ();
}



void YVE_StringEditorEventHandler::_InvertCharRange
    (UI_DrawingSurface* sfc, const CL_Interval& sel)
{
    CL_String& data = (CL_String&) _edit->Model();
    short fontHeight = sfc->Font().Height();
    if (sel.High() >= 0) {
        XFontStruct* fontPtr = (XFontStruct*) sfc->Font().NativeFontStruct();
        const char* dataPtr = data.AsPtr();
        long xLeft = XTextWidth
            (fontPtr, dataPtr, maxl (sel.Low(), 0));
        long hi = sel.High() + 1;
        long xRight = XTextWidth (fontPtr, dataPtr, hi);
        sfc->InvertRectangle (UI_Rectangle (xLeft+2, 3, xRight - xLeft,
                                             fontHeight-1));
    }
}


bool YVE_StringEditorEventHandler::_BlinkTimerTick ()
{
    _blinkOn = 1 - _blinkOn;
    _DoPaint ();
    return TRUE;
}

