



#if defined(__GNUC__) && __GNUC_MINOR__ >= 7
#    include <string.h>  // Without this, the X includes barf
#endif

#include <X11/Xlib.h>
#include <sys/types.h>
#include <sys/time.h>
#if defined(__SGI_DELTA_CC__)
#    include <unistd.h>
#endif

#include <iostream.h> // DEBUG

#include "base/genseq.h"

#include "ui/cntroler.h"
#include "ui/applic.h"



class TimerRequest: public CL_Object {

public:
    TimerRequest (ulong id, ulong mSec, UI_Timer* timer);

    ulong      _id;        // Timer id
    UI_Timer*  _timer;     // Corresponding Timer object
    timeval    _expTime; // Time at which this timer will expire
};


ostream& operator<< (ostream& s, const timeval& t)
{  /// DEBUG only
    s << t.tv_sec << " " << t.tv_usec;
    return s;
}

TimerRequest::TimerRequest (ulong id, ulong mSec, UI_Timer* timer)
: _id (id), _timer (timer)
{
    gettimeofday (&_expTime, NULL);
    _expTime.tv_usec += mSec * 1000;
    _expTime.tv_sec += _expTime.tv_usec / 1000000;
    _expTime.tv_usec %= 1000000;
}


static timeval _Subtract (const timeval& t1, const timeval& t2)
{
    // Return the result of subtracting t2 from t1
    timeval r = {t1.tv_sec - t2.tv_sec, t1.tv_usec - t2.tv_usec};
    if (r.tv_usec < 0) {
        r.tv_usec += 1000000;
        r.tv_sec --;
    }
    return r;
}

bool UI_Controller::_WaitForNextEvent ()
{
    fd_set fds;
    int fd = ConnectionNumber (_display);
    FD_SET (fd, &fds);
    long timerQSize = _timerQueue.Size();
    if (timerQSize <= 0) {
        // No timer requests pending
        // static timeval zeroTime = {0};
        select (fd + 1, &fds, NULL, NULL, NULL);
        return FALSE;
    }
    timeval now;
    gettimeofday (&now, NULL);
    timeval tval;
    TimerRequest* rq = (TimerRequest*) _timerQueue (0);
    tval = _Subtract (rq->_expTime, now);
    if (tval.tv_sec < 0 || tval.tv_sec == 0 && tval.tv_usec <= 0)
        return TRUE;
    int result = select (fd + 1, &fds, NULL, NULL, &tval);
    return result == 0;
}

ulong UI_Controller::RegisterTimeOut (long msec,  UI_Timer* timer)
{
    if (msec > 0) {
        ulong id = _nextTimerId++;
        _timerQueue.Add (new TimerRequest (id, msec, timer));
        return id;
    }
    return 0;
}

void UI_Controller::UnregisterTimeOut (ulong id)
{
    long i;
    TimerRequest* rq = NULL;
    for (i = _timerQueue.Size() - 1; i >= 0; i--) {
        rq = (TimerRequest*) _timerQueue(i);
        if (rq->_id == id)
            break;
    }
    if (i >= 0) {
        _timerQueue.Remove (i);
        delete rq;
    }
}


void UI_Controller::_DispatchTimerEvents ()
{
    timeval now;
    gettimeofday (&now, NULL);
    timeval tval;
    while (_timerQueue.Size()) {
        TimerRequest* rq = (TimerRequest*) _timerQueue (0);
        tval = _Subtract (rq->_expTime, now);
        if (tval.tv_sec < 0 || tval.tv_sec == 0 && tval.tv_usec <= 0) {
            _timerQueue.ShiftLeftAt (1);
            UI_Timer::DoAlarm (rq->_timer);
            delete rq;
        }
        else
            break;
    }
}

